-- 
-- This file is automatically generated by Cardpeek.
-- It holds cardpeek configuration parameters, which are stored in the
-- lua 'cardpeek' table.
-- 
-- See cardpeekrc.lua for adding your own functions and variables to
-- cardpeek.
-- 
cardpeek = {
}

dofile('scripts/lib/apdu.lua')
-- end --
