BANLIEUE_LIST = {
  [01] = {
    [00] = "Châtelet-Les Halles",
    [01] = "Châtelet-Les Halles",
    [07] = "Luxembourg"
  },
  [03] = {
    [00] = "Saint-Michel Notre-Dame"
  },
  [06] = {
    [00] = "Auber"
  },
  [14] = {
    [04] = "Cité Universitaire"
  },
  [15] = {
    [12] = "Port Royal",
    [13] = "Denfert-Rochereau"
  },
  [16] = {
    [01] = "Nation",
    [02] = "Fontenay-sous-Bois|Vincennes",
    [03] = "Joinville-le-Pont|Nogent-sur-Marne",
    [04] = "Saint-Maur Créteil",
    [05] = "Le Parc de Saint-Maur",
    [06] = "Champigny",
    [07] = "La Varenne-Chennevières",
    [08] = "Boissy-Saint-Léger|Sucy Bonneuil"
  },
  [17] = {
    [01] = "Charles de Gaulle-Etoile",
    [04] = "La Défense (Grande Arche)",
    [05] = "Nanterre-Université|Nanterre-Ville|Nanterre-Préfecture",
    [06] = "Rueil-Malmaison",
    [08] = "Chatou-Croissy",
    [09] = "Le Vésinet-Centre|Le Vésinet-Le Pecq|Saint-Germain-en-Laye"
  },
  [18] = {
    [00] = "Denfert-Rochereau",
    [01] = "Gentilly",
    [02] = "Arcueil-Cachan|Laplace",
    [03] = "Bagneux|Bourg-la-Reine",
    [04] = "La Croix-de-Berny|Parc de Sceaux",
    [05] = "Antony|Fontaine-Michalon|Les Baconnets",
    [06] = "Massy-Palaiseau|Massy-Verrières",
    [07] = "Palaiseau|Palaiseau Villebon",
    [08] = "Lozère",
    [09] = "Le Guichet|Orsay-Ville",
    [10] = "Bures-sur-Yvette|Courcelle-sur-Yvette|Gif-sur-Yvette|La Hacquinière|Saint-Rémy-lès-Chevreuse"
  },
  [20] = {
    [01] = "Gare de l'Est",
    [04] = "Pantin",
    [05] = "Noisy-le-Sec",
    [06] = "Bondy",
    [07] = "Gagny|Le Raincy Villemomble",
    [09] = "Chelles Gournay|Le Chênay Gagny",
    [10] = "Vaires Torcy",
    [11] = "Lagny-Thorigny",
    [13] = "Esbly",
    [14] = "Meaux",
    [15] = "Changis-Saint-Jean|Isles-Armentières Congis|Lizy-sur-Ourcq|Trilport",
    [16] = "Crouy-sur-Ourcq|La Ferté-sous-Jouarre|Nanteuil Saacy"
  },
  [21] = {
    [05] = "Rosny-Bois-Perrier|Rosny-sous-Bois|Val de Fontenay",
    [06] = "Nogent Le-Perreux",
    [07] = "Les Boullereaux Champigny",
    [08] = "Villiers-sur-Marne",
    [09] = "Les Yvris Noisy-le-Grand",
    [10] = "Emerainville|Roissy-en-Brie",
    [11] = "Ozoir-la-Ferrière",
    [12] = "Gretz-Armainvilliers|Tournan",
    [15] = "Courquetaine|Faremoutiers Pommeuse|Guerard La-Celle-sur-Morin|Liverdy en Brie|Marles-en-Brie|Mormant|Mortcerf|Mouroux|Ozouer le voulgis|Verneuil-l'Etang|Villepatour - Presles|Yebles|Yebles - Guignes",
    [16] = "Chailly Boissy-le-Châtel|Chauffry|Coulommiers|Jouy-sur-Morin Le-Marais|Nangis|Saint-Rémy-la-Vanne|Saint-Siméon",
    [17] = "Champbenoist-Poigny|La Ferté-Gaucher|Longueville|Provins|Sainte-Colombe-Septveilles",
    [18] = "Flamboin|Meilleray|Villiers St Georges"
  },
  [22] = {
    [07] = "Allée de la Tour-Rendez-Vous|La Remise-à-Jorelle|Les Coquetiers|Les Pavillons-sous-Bois",
    [08] = "Gargan",
    [09] = "Freinville Sevran|L'Abbaye|Lycée Henri Sellier|Rougemont-Chanteloup"
  },
  [23] = {
    [13] = "Couilly Saint-Germain Quincy|Les Champs-Forts|Montry Condé",
    [14] = "Crécy-en-Brie La Chapelle|Villiers-Montbarbin"
  },
  [26] = {
    [05] = "Val de Fontenay",
    [06] = "Bry-sur-Marne|Neuilly-Plaisance",
    [07] = "Noisy-le-Grand (Mont d'Est)",
    [08] = "Noisy-Champs",
    [10] = "Lognes|Noisiel|Torcy",
    [11] = "Bussy-Saint-Georges",
    [12] = "Val d'europe",
    [13] = "Marne-la-Vallée Chessy"
  },
  [28] = {
    [04] = "Fontenay-aux-Roses|Robinson|Sceaux"
  },
  [30] = {
    [01] = "Gare Saint-Lazare",
    [03] = "Pont Cardinet",
    [04] = "Asnières|Bécon-les-Bruyères|Clichy Levallois|Courbevoie|La Défense (Grande Arche)",
    [05] = "Puteaux|Suresnes Mont-Valérien",
    [07] = "Garches Marne-la-Coquette|Le Val d'Or|Saint-Cloud",
    [08] = "Vaucresson",
    [09] = "Bougival|La Celle-Saint-Cloud|Louveciennes|Marly-le-Roi",
    [10] = "L'Etang-la-Ville|Saint-Nom-la-Bretêche Forêt"
  },
  [31] = {
    [07] = "Chaville-Rive Droite|Sèvres Ville-d'Avray|Viroflay-Rive Droite",
    [08] = "Montreuil|Versailles-Rive Droite"
  },
  [32] = {
    [05] = "La Garenne-Colombes|Les Vallées|Nanterre-Université",
    [07] = "Houilles Carrières-sur-Seine|Sartrouville",
    [09] = "Maisons-Laffitte",
    [10] = "Poissy",
    [11] = "Médan|Villennes-sur-Seine",
    [12] = "Les Clairières de Verneuil|Vernouillet Verneuil",
    [13] = "Aubergenville-Elisabethville|Les Mureaux",
    [14] = "Epône Mézières",
    [16] = "Bonnières|Mantes-la-Jolie|Mantes-Station|Port-Villez|Rosny-sur-Seine"
  },
  [33] = {
    [10] = "Achères-Grand-Cormier|Achères-Ville",
    [11] = "Cergy-Préfecture|Neuville-Université",
    [12] = "Cergy-le-Haut|Cergy-Saint-Christophe"
  },
  [34] = {
    [05] = "Suresnes - Lonchamp",
    [06] = "Les Côteaux|Pont de Saint Cloud|Pont de Sèvres",
    [07] = "Bas Meudon|Bellevue funiculaire",
    [08] = "Les Moulineaux - Billancourt"
  },
  [35] = {
    [04] = "Bois-Colombes",
    [05] = "Colombes|Le Stade",
    [06] = "Argenteuil",
    [08] = "Cormeilles-en-Parisis|Val d'Argenteuil",
    [09] = "Herblay|La Frette Montigny",
    [10] = "Conflans-Fin d'Oise|Conflans-Sainte-Honorine",
    [11] = "Andrésy|Chanteloup-lès-Vignes|Maurecourt",
    [12] = "Triel-sur-Seine|Vaux-sur-Seine",
    [13] = "Meulan Hadricourt|Thun-le-Paradis",
    [14] = "Gargenville|Juziers",
    [15] = "Issou Porcheville|Limay",
    [16] = "Breval|Ménerville"
  },
  [40] = {
    [01] = "Gare de Lyon|Gare de Lyon (Banlieue)",
    [05] = "Le Vert de Maisons|Maisons-Alfort Alfortville",
    [06] = "Villeneuve-Prairie",
    [07] = "Villeneuve-Triage",
    [08] = "Villeneuve-Saint-Georges",
    [09] = "Juvisy|Vigneux-sur-Seine",
    [10] = "Ris-Orangis|Viry-Châtillon",
    [11] = "Evry Val de Seine|Grand-Bourg",
    [12] = "Corbeil-Essonnes|Mennecy|Moulin-Galant",
    [13] = "Ballancourt|Fontenay le Vicomte",
    [14] = "La Ferté-Alais",
    [16] = "Boutigny|Maisse",
    [17] = "Boigneville|Buno-Gironville"
  },
  [41] = {
    [00] = "Saint-Michel Notre-Dame|Musée d'Orsay",
    [01] = "Gare d'Austerlitz",
    [02] = "Bibliotheque-Francois",
    [03] = "Boulevard Masséna",
    [04] = "Ivry-sur-Seine|Vitry-sur-Seine",
    [05] = "Choisy-le-Roi|Les Ardoines",
    [07] = "Villeneuve-le-Roi",
    [08] = "Ablon",
    [09] = "Athis-Mons"
  },
  [42] = {
    [09] = "Epinay-sur-Orge|Savigny-sur-Orge",
    [10] = "Sainte-Geneviève-des-Bois",
    [11] = "Saint-Michel-sur-Orge",
    [12] = "Brétigny-sur-Orge|Marolles-en-Hurepoix",
    [13] = "Bouray|Lardy",
    [14] = "Chamarande|Etampes|Etréchy",
    [16] = "Saint-Martin d'Etampes",
    [17] = "Guillerval"
  },
  [43] = {
    [09] = "Montgeron Crosne|Yerres",
    [10] = "Brunoy",
    [11] = "Boussy-Saint-Antoine|Combs-la-Ville Quincy",
    [12] = "Lieusaint Moissy",
    [13] = "Cesson|Savigny-le-Temple Nandy",
    [15] = "Le Mée|Melun",
    [16] = "Chartrettes|Fontaine-le-Port|Livry-sur-Seine",
    [17] = "Champagne-sur-Seine|Héricy|La Grande Paroisse|Vernou-sur-Seine|Vulaines-sur-Seine Samoreau"
  },
  [44] = {
    [12] = "Essonnes-Robinson|Villabé",
    [13] = "Coudray-Montceaux|Le Plessis-Chenet-IBM|Saint-Fargeau",
    [14] = "Boissise-le-Roi|Ponthierry Pringy",
    [15] = "Vosves",
    [16] = "Bois-le-Roi",
    [17] = "Bagneaux-sur-Loing|Bourron-Marlotte Grez|Fontainebleau-Avon|Montereau|Montigny-sur-Loing|Moret Veneux-les-Sablons|Nemours Saint-Pierre|Saint-Mammés|Souppes|Thomery"
  },
  [45] = {
    [10] = "Grigny-Centre",
    [11] = "Evry Courcouronnes|Orangis Bois de l'Epine",
    [12] = "Le Bras-de-Fer - Evry"
  },
  [50] = {
    [00] = "Haussmann-Saint-Lazare",
    [01] = "Gare du Nord|Gare du Nord (Surface)|Magenta|Paris-Nord",
    [05] = "Epinay-Villetaneuse|Saint-Denis|Sèvres-Rive Gauche",
    [06] = "La Barre-Ormesson",
    [07] = "Champ de Courses d'Enghien|Enghien-les-Bains",
    [08] = "Ermont Eaubonne|Ermont-Eaubonne|Ermont-Halte|Gros-Noyer Saint-Prix",
    [09] = "Saint-Leu-La-Forêt|Taverny|Vaucelles",
    [10] = "Bessancourt|Frépillon|Mery",
    [11] = "Mériel|Valmondois",
    [12] = "Bruyères-sur-Oise|Champagne-sur-Oise|L'Isle-Adam Parmain|Persan Beaumont"
  },
  [51] = {
    [04] = "La Courneuve-Aubervilliers|La Plaine-Stade de France",
    [05] = "Le Bourget",
    [07] = "Blanc-Mesnil|Drancy",
    [08] = "Aulnay-sous-Bois",
    [09] = "Sevran Livry|Vert-Galant",
    [10] = "Villeparisis",
    [11] = "Compans|Mitry Claye|Mitry-Claye",
    [12] = "Dammartin Juilly Saint-Mard|Thieux Nantouillet"
  },
  [52] = {
    [05] = "Stade de France-Saint-Denis",
    [06] = "Pierrefitte Stains",
    [07] = "Garges-Sarcelles",
    [08] = "Villiers-le-Bel (Gonesse -",
    [10] = "Goussainville|Les Noues|Louvres",
    [11] = "La Borne-Blanche|Survilliers-Fosses"
  },
  [53] = {
    [06] = "Deuil Montmagny",
    [07] = "Groslay",
    [08] = "Sarcelles Saint-Brice",
    [09] = "Domont|Ecouen Ezanville",
    [10] = "Bouffemont Moisselles|Montsoult Maffliers",
    [11] = "Belloy-Saint-Martin|Luzarches|Seugy|Viarmes|Villaines"
  },
  [54] = {
    [08] = "Cernay",
    [09] = "Franconville|Montigny Beauchamp|Montigny-Beauchamp",
    [10] = "Pierrelaye",
    [11] = "Pontoise|Saint-Ouen-l'Aumône|Saint-Ouen-l'Aumone-Liesse",
    [12] = "Boissy-l'Aillerie|Osny",
    [15] = "Chars|Montgeroult Courcelles|Santeuil Le Perchay|Us"
  },
  [55] = {
    [00] = "Avenue Foch|Avenue Henri-Martin|Kennedy Radio-France|Boulainvilliers|Neuilly-Porte Maillot",
    [01] = "Péreire-Levallois",
    [02] = "Porte de Clichy",
    [03] = "Saint-Ouen",
    [04] = "Les Grésillons",
    [05] = "Gennevilliers",
    [06] = "Epinay-sur-Seine",
    [07] = "Saint-Gratien"
  },
  [56] = {
    [11] = "Auvers-sur-Oise|Chaponval|Epluches|Pont Petit"
  },
  [57] = {
    [11] = "Presles Courcelles",
    [12] = "Nointel Mours"
  },
  [60] = {
    [01] = "Gare Montparnasse|Paris Montparnasse3 Vaug.",
    [04] = "Clamart|Vanves Malakoff",
    [05] = "Bellevue|Bièvres|Meudon",
    [06] = "Chaville-Rive Gauche|Chaville-Vélizy|Viroflay-Rive Gauche",
    [07] = "Versailles-Chantiers",
    [10] = "Saint-Cyr",
    [11] = "Saint-Quentin-en-Yvelines -|Trappes",
    [12] = "Coignières|La Verrière",
    [13] = "Les Essarts-le-Roi",
    [14] = "Le Perray|Rambouillet",
    [15] = "Gazeran"
  },
  [61] = {
    [10] = "Fontenay-le-Fleury",
    [11] = "Villepreux Les-Clayes",
    [12] = "Plaisir Grignon|Plaisir Les-Clayes",
    [13] = "Beynes|Mareil-sur-Mauldre|Maule|Nézel Aulnay",
    [15] = "Garancières La-Queue|Montfort-l'Amaury Méré|Orgerus Behoust|Tacoignères Richebourg|Villiers Neauphle",
    [16] = "Houdan"
  },
  [63] = {
    [07] = "Porchefontaine|Versailles-Rive Gauche"
  },
  [64] = {
    [00] = "Pont de l'alma|Invalides",
    [01] = "Champ de Mars-Tour Eiffel",
    [02] = "Javel",
    [03] = "Boulevard Victor - Pont du Garigliano",
    [04] = "Issy-Val de Seine",
    [05] = "Issy|Meudon-Val-Fleury"
  },
  [65] = {
    [08] = "Jouy-en-Josas|Petit-Jouy-les-Loges",
    [09] = "Vauboyen",
    [10] = "Igny",
    [11] = "Massy-Palaiseau",
    [12] = "Longjumeau",
    [13] = "Chilly-Mazarin",
    [14] = "Gravigny-Balizy|Petit-Vaux"
  },
  [70] = {
    [09] = "Parc des Expositions|Sevran-Beaudottes|Villepinte",
    [10] = "Aéroport Charles de Gaulle 1|Aéroport Charles de Gaulle 2"
  },
  [72] = {
    [07] = "Sannois"
  },
  [73] = {
    [11] = "Eragny Neuville|Saint-Ouen-l'Aumône (Eglise)"
  },
  [75] = {
    [07] = "Les Saules|Orly-Ville",
    [09] = "Pont de Rungis Aéroport|Rungis-La Fraternelle",
    [10] = "Chemin d'Antony",
    [12] = "Massy-Verrières"
  },
  [76] = {
    [12] = "Arpajon|Egly|La Norville",
    [13] = "Breuillet Bruyères-le-Châtel|Breuillet-Village|Saint-Chéron",
    [14] = "Sermaise",
    [15] = "Dourdan|Dourdan-la-Forêt"
  }
}

BANLIEUE_NET_LIST = {
  [1] = "RATP",
  [2] = "Transilien Paris Est",
  [3] = "Transilien Paris Saint-Lazare",
  [4] = "Transilien Paris Lyon",
  [5] = "Transilien Paris Nord",
  [6] = "Transilien Paris Montparnasse"
}

