METRO_LIST = {
[01] = {
  ["name"] = "Cité",
  [01] = "Saint-Michel",
  [04] = "Odéon",
  [05] = "Cluny - La Sorbonne",
  [06] = "Maubert - Mutualité",
  [07] = "Luxembourg",
  [08] = "Châtelet",
  [09] = "Les Halles",
  [10] = "Les Halles",
  [12] = "Louvre - Rivoli",
  [13] = "Pont Neuf",
  [14] = "Cité",
  [15] = "Hôtel de Ville"
},
[02] = {
  ["name"] = "Rennes",
  [02] = "Cambronne",
  [03] = "Sèvres - Lecourbe",
  [04] = "Ségur",
  [06] = "Saint-François-Xavier",
  [07] = "Duroc",
  [08] = "Vaneau",
  [09] = "Sèvres - Babylone",
  [10] = "Rue du Bac",
  [11] = "Rennes",
  [12] = "Saint-Sulpice",
  [14] = "Mabillon",
  [15] = "Saint-Germain-des-Prés"
},
[03] = {
  ["name"] = "Villette",
  [04] = "Porte de la Villette",
  [05] = "Aubervilliers - Pantin - Quatre Chemins",
  [06] = "Fort d'Aubervilliers",
  [07] = "La Courneuve - 8 Mai 1945",
  [09] = "Hoche",
  [10] = "Église de Pantin",
  [11] = "Bobigny - Pantin - Raymond Queneau",
  [12] = "Bobigny - Pablo Picasso"
},
[04] = {
  ["name"] = "Montparnasse",
  [02] = "Pernety",
  [03] = "Plaisance",
  [04] = "Gaîté",
  [06] = "Edgar Quinet",
  [07] = "Vavin",
  [08] = "Montparnasse - Bienvenüe",
  [12] = "Saint-Placide",
  [14] = "Notre-Dame-des-Champs"
},
[05] = {
  ["name"] = "Nation",
  [02] = "Robespierre",
  [03] = "Porte de Montreuil",
  [04] = "Maraîchers",
  [05] = "Buzenval",
  [06] = "Rue des Boulets",
  [07] = "Porte de Vincennes",
  [09] = "Picpus",
  [10] = "Nation",
  [12] = "Avron",
  [13] = "Alexandre Dumas"
},
[06] = {
  ["name"] = "Saint-Lazare",
  [01] = "Malesherbes",
  [02] = "Monceau",
  [03] = "Villiers",
  [04] = "Quatre-Septembre",
  [05] = "Opéra",
  [06] = "Auber",
  [07] = "Havre - Caumartin",
  [08] = "Saint-Lazare",
  [09] = "Saint-Lazare",
  [10] = "Saint-Augustin",
  [12] = "Europe",
  [13] = "Liège"
},
[07] = {
  ["name"] = "Auteuil",
  [03] = "Porte de Saint-Cloud",
  [07] = "Porte d'Auteuil",
  [08] = "Église d'Auteuil",
  [09] = "Michel-Ange - Auteuil",
  [10] = "Michel-Ange - Molitor",
  [11] = "Chardon-Lagache",
  [12] = "Mirabeau",
  [14] = "Exelmans",
  [15] = "Jasmin"
},
[08] = {
  ["name"] = "République",
  [01] = "Rambuteau",
  [03] = "Arts et Métiers",
  [04] = "Jacques Bonsergent",
  [05] = "Goncourt",
  [06] = "Temple",
  [07] = "République",
  [10] = "Oberkampf",
  [11] = "Parmentier",
  [12] = "Filles du Calvaire",
  [13] = "Saint-Sébastien - Froissart",
  [14] = "Richard-Lenoir",
  [15] = "Saint-Ambroise"
},
[09] = {
  ["name"] = "Austerlitz",
  [01] = "Quai de la Gare",
  [02] = "Chevaleret",
  [04] = "Saint-Marcel",
  [07] = "Gare d'Austerlitz",
  [08] = "Gare de Lyon",
  [10] = "Quai de la Rapée"
},
[10] = {
  ["name"] = "Invalides",
  [01] = "Champs-Élysées - Clemenceau",
  [02] = "Concorde",
  [03] = "Madeleine",
  [04] = "Bir-Hakeim",
  [07] = "École Militaire",
  [08] = "La Tour-Maubourg",
  [09] = "Invalides",
  [11] = "Saint-Denis - Université",
  [12] = "Varenne",
  [13] = "Assemblée nationale",
  [14] = "Solférino"
},
[11] = {
  ["name"] = "Sentier",
  [01] = "Tuileries",
  [02] = "Palais Royal - Musée du Louvre",
  [03] = "Pyramides",
  [04] = "Bourse",
  [06] = "Grands Boulevards",
  [07] = "Richelieu - Drouot",
  [08] = "Bonne Nouvelle",
  [10] = "Strasbourg - Saint-Denis",
  [11] = "Château d'Eau",
  [13] = "Sentier",
  [14] = "Réaumur - Sébastopol",
  [15] = "Étienne Marcel"
},
[12] = {
  ["name"] = "Île Saint-Louis",
  [01] = "Faidherbe - Chaligny",
  [02] = "Reuilly - Diderot",
  [03] = "Montgallet",
  [04] = "Censier - Daubenton",
  [05] = "Place Monge",
  [06] = "Cardinal Lemoine",
  [07] = "Jussieu",
  [08] = "Sully - Morland",
  [09] = "Pont Marie",
  [10] = "Saint-Paul",
  [12] = "Bastille",
  [13] = "Chemin Vert",
  [14] = "Bréguet - Sabin",
  [15] = "Ledru-Rollin"
},
[13] = {
  ["name"] = "Daumesnil",
  [01] = "Porte Dorée",
  [03] = "Porte de Charenton",
  [07] = "Bercy",
  [08] = "Dugommier",
  [10] = "Michel Bizot",
  [11] = "Daumesnil",
  [12] = "Bel-Air"
},
[14] = {
  ["name"] = "Italie",
  [02] = "Porte de Choisy",
  [03] = "Porte d'Italie",
  [04] = "Cité universitaire",
  [09] = "Maison Blanche",
  [10] = "Tolbiac",
  [11] = "Nationale",
  [12] = "Campo-Formio",
  [13] = "Les Gobelins",
  [14] = "Place d'Italie",
  [15] = "Corvisart"
},
[15] = {
  ["name"] = "Denfert",
  [01] = "Cour Saint-Emilion",
  [02] = "Porte d'Orléans",
  [03] = "Bibliothèque François Mitterrand",
  [04] = "Mouton-Duvernet",
  [05] = "Alésia",
  [06] = "Olympiades",
  [08] = "Glacière",
  [09] = "Saint-Jacques",
  [10] = "Raspail",
  [14] = "Denfert-Rochereau"
},
[16] = {
  ["name"] = "Félix Faure",
  [01] = "Falguière",
  [02] = "Pasteur",
  [03] = "Volontaires",
  [04] = "Vaugirard",
  [05] = "Convention",
  [06] = "Porte de Versailles",
  [09] = "Balard",
  [10] = "Lourmel",
  [11] = "Boucicaut",
  [12] = "Félix Faure",
  [13] = "Charles Michels",
  [14] = "Javel - André Citroën"
},
[17] = {
  ["name"] = "Passy",
  [02] = "Porte Dauphine",
  [04] = "La Motte-Picquet - Grenelle",
  [05] = "Commerce",
  [06] = "Avenue Émile Zola",
  [07] = "Dupleix",
  [08] = "Passy",
  [09] = "Ranelagh",
  [11] = "La Muette",
  [13] = "Rue de la Pompe",
  [14] = "Boissière",
  [15] = "Trocadéro"
},
[18] = {
  ["name"] = "Étoile",
  [01] = "Iéna",
  [03] = "Alma - Marceau",
  [04] = "Miromesnil",
  [05] = "Saint-Philippe du Roule",
  [07] = "Franklin D. Roosevelt",
  [08] = "George V",
  [09] = "Kléber",
  [10] = "Victor Hugo",
  [11] = "Argentine",
  [12] = "Charles de Gaulle - ?~Itoile",
  [14] = "Ternes",
  [15] = "Courcelles"
},
[19] = {
  ["name"] = "Clichy - Saint Ouen",
  [01] = "Mairie de Clichy",
  [02] = "Gabriel Péri",
  [03] = "Les Agnettes",
  [04] = "Asnières - Gennevilliers - Les Courtilles",
  [09] = "La Chapelle)",
  [10] = "Garibaldi",
  [11] = "Mairie de Saint-Ouen",
  [13] = "Carrefour Pleyel",
  [14] = "Saint-Denis - Porte de Paris",
  [15] = "Basilique de Saint-Denis"
},
[20] = {
  ["name"] = "Montmartre",
  [01] = "Porte de Clignancourt",
  [06] = "Porte de la Chapelle",
  [07] = "Marx Dormoy",
  [09] = "Marcadet - Poissonniers",
  [10] = "Simplon",
  [11] = "Jules Joffrin",
  [12] = "Lamarck - Caulaincourt"
},
[21] = {
  ["name"] = "Lafayette",
  [01] = "Chaussée d'Antin - La Fayette",
  [02] = "Le Peletier",
  [03] = "Cadet",
  [04] = "Château Rouge",
  [07] = "Barbès - Rochechouart",
  [08] = "Gare du Nord",
  [09] = "Gare de l'Est",
  [10] = "Poissonnière",
  [11] = "Château-Landon"
},
[22] = {
  ["name"] = "Buttes Chaumont",
  [01] = "Porte de Pantin",
  [02] = "Ourcq",
  [04] = "Corentin Cariou",
  [06] = "Crimée",
  [08] = "Riquet",
  [09] = "La Chapelle",
  [10] = "Louis Blanc",
  [11] = "Stalingrad",
  [12] = "Jaurès",
  [13] = "Laumière",
  [14] = "Bolivar",
  [15] = "Colonel Fabien"
},
[23] = {
  ["name"] = "Belleville",
  [02] = "Porte des Lilas",
  [03] = "Mairie des Lilas",
  [04] = "Porte de Bagnolet",
  [05] = "Gallieni",
  [08] = "Place des Fêtes",
  [09] = "Botzaris",
  [10] = "Danube",
  [11] = "Pré Saint-Gervais",
  [13] = "Buttes Chaumont",
  [14] = "Jourdain",
  [15] = "Télégraphe"
},
[24] = {
  ["name"] = "Père Lachaise",
  [01] = "Voltaire",
  [02] = "Charonne",
  [04] = "Père Lachaise",
  [05] = "Ménilmontant",
  [06] = "Rue Saint-Maur",
  [07] = "Philippe Auguste",
  [08] = "Saint-Fargeau",
  [09] = "Pelleport",
  [10] = "Gambetta",
  [12] = "Belleville",
  [13] = "Couronnes",
  [14] = "Pyrénées"
},
[25] = {
  ["name"] = "Charenton",
  [02] = "Croix de Chavaux",
  [03] = "Mairie de Montreuil",
  [04] = "Maisons-Alfort - Les Juilliottes",
  [05] = "Créteil - L'Échat",
  [06] = "Créteil - Université",
  [07] = "Créteil - Préfecture",
  [08] = "Saint-Mandé",
  [10] = "Bérault",
  [11] = "Château de Vincennes",
  [12] = "Liberté",
  [13] = "Charenton - Écoles",
  [14] = "École vétérinaire de Maisons-Alfort",
  [15] = "Maisons-Alfort - Stade"
},
[26] = {
  ["name"] = "Ivry - Villejuif",
  [03] = "Porte d'Ivry",
  [04] = "Pierre et Marie Curie",
  [05] = "Mairie d'Ivry",
  [06] = "Le Kremlin-Bicêtre",
  [07] = "Villejuif - Léo Lagrange",
  [08] = "Villejuif - Paul Vaillant-Couturier",
  [09] = "Villejuif - Louis Aragon"
},
[27] = {
  ["name"] = "Vanves",
  [02] = "Porte de Vanves",
  [07] = "Malakoff - Plateau de Vanves",
  [08] = "Malakoff - Rue Étienne Dolet",
  [09] = "Châtillon - Montrouge"
},
[28] = {
  ["name"] = "Issy",
  [02] = "Corentin Celton",
  [03] = "Mairie d'Issy",
  [08] = "Marcel Sembat",
  [09] = "Billancourt",
  [10] = "Pont de Sèvres"
},
[29] = {
  ["name"] = "Levallois",
  [00] = "La Défense (correspondance RER)",
  [04] = "Boulogne - Jean Jaurès",
  [05] = "Boulogne - Pont de Saint-Cloud",
  [08] = "Les Sablons",
  [09] = "Pont de Neuilly",
  [10] = "Esplanade de la Défense",
  [11] = "La Défense",
  [12] = "Porte de Champerret",
  [13] = "Louise Michel",
  [14] = "Anatole France",
  [15] = "Pont de Levallois - Bécon"
},
[30] = {
  ["name"] = "Péreire",
  [01] = "Porte Maillot",
  [04] = "Wagram",
  [05] = "Pereire",
  [08] = "Brochant",
  [09] = "Porte de Clichy",
  [12] = "Guy Môquet",
  [13] = "Porte de Saint-Ouen"
},
[31] = {
  ["name"] = "Pigalle",
  [02] = "Funiculaire de Montmartre (station inférieure)",
  [03] = "Funiculaire de Montmartre (station supérieure)",
  [04] = "Anvers",
  [05] = "Abbesses",
  [06] = "Pigalle",
  [07] = "Blanche",
  [08] = "Trinité - d'Estienne d'Orves",
  [09] = "Notre-Dame-de-Lorette",
  [10] = "Saint-Georges",
  [12] = "Rome",
  [13] = "Place de Clichy",
  [14] = "La Fourche"
}
}
